@@ -14,10 +14,8 @@ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
  """
-import logging
 
 from .yamlHandler import YamlHandler
-from .log4Vol2Timing import Loggings
 from .Enums import *
 import os
 import pandas as pd
@@ -27,9 +25,35 @@ import datetime
 from .intermediateFiles import *
 from .coder import *
 
-loggings = Loggings()
-loggingTimeInfo = "Set Logging System: " + str(datetime.datetime.now())
-loggings.info(loggingTimeInfo, 0)
+
+mesonetPath=""
+micronetPath=""
+movementFileName=""
+macroLinkFileName=""
+macroNodeFileName=""
+
+mesoLinkFileName=""
+mesoNodeFileName=""
+
+microLinkFileName=""
+microLinkBackupFileName=""
+microNodeFileName=""
+def set_map_folder(root):
+    global mesonetPath,micronetPath,movementFileName,macroLinkFileName,macroNodeFileName,mesoLinkFileName,mesoNodeFileName
+    global microLinkFileName,microNodeFileName,microLinkBackupFileName
+    mesonetPath = os.path.join(root, "mesonet")
+    micronetPath = os.path.join(root, "micronet")
+    movementFileName = os.path.join(root, "movement.csv")
+    macroLinkFileName = os.path.join(root, "link.csv")
+    macroNodeFileName = os.path.join(root, "node.csv")
+
+    mesoLinkFileName = os.path.join(mesonetPath, "link.csv")
+    mesoNodeFileName = os.path.join(mesonetPath, "node.csv")
+
+    microLinkFileName = os.path.join(micronetPath, "link.csv")
+    microLinkBackupFileName = os.path.join(micronetPath, "link_backup.csv")
+    microNodeFileName = os.path.join(micronetPath, "node.csv")
+
 
 
 g_node_vector = []
@@ -37,7 +61,6 @@ g_link_vector = []
 g_node_map = {}
 g_link_map={}
 g_inter=0
-loggings.info("Set Global Variable")
 
 # array size, for constructing matirx or array
 laneColumnSize = 32
@@ -47,14 +70,13 @@ NEMA_PhaseSize = 32  # temp enable=false
 stageSize = 4
 ringSize = 3
 groupSize = 5
+saturated_flow_rate=1900
+
 
-loggings.info("Set Array, Matrix, Dictionary Size")
-loggings.info("Initialization Completed!", 0)
 
 
 class CMainModual:
     def __init__(self):
-        loggings.info("Build Main Module", 0)
         self.g_number_of_links = 0
         self.g_number_of_nodes = 0
         self.b_debug_detail_flag = 1
@@ -97,9 +119,7 @@ class CMovementData:
 
 class CSignalNode:
     def __init__(self, nodeID, xcoord, ycoord,_config_Parameters,reference_cycle_length):
-        loggings.info("Set Parameters for Signal Node")
-        loggings.info("Checking .yaml file")
-        loggings.info(f"Main Node ID: {str(nodeID)}",2)
+
         self.nodeID=nodeID
         self.xcoord=xcoord
         self.ycoord=ycoord
@@ -327,11 +347,8 @@ class CSignalNode:
         self.movement_Array[index].SharedLanes = sharedLanes
         self.movement_Array[index].Geometry=geometry
         self.movement_Array[index].mvmt_id=mvmt_id
-        self.Info=f"Add Movement: {mi.name}, Attributes: link_id(in:{ib_link_id},out:{ob_link_id}), Volume({volume}), Lanes({lanes})"
-        loggings.info(self.Info,3)
 
     def Set_Left_Turn_Treatment(self):
-        loggings.info("Set Left Turn Treatment",3)
         for m in range(1, movementSize + 1):
             if not self.movement_Array[m].Enable:
                 continue
@@ -365,9 +382,7 @@ class CSignalNode:
                 final_decision = -1
 
             self.movement_Array[m].Left_Turn_Treatment = ELeft_Turn_Treatment(final_decision)
-            if final_decision!=-1:
-                self.Info = f"Movement: {EMovement_Index(m).name}, Attributes: ELeft_Turn_Treatment({ELeft_Turn_Treatment(final_decision).name})"
-                loggings.info(self.Info, 4)
+
     def Set_Major_Apporach(self):
         east_And_West_Volume = self.movement_Array[EMovement_Index.EBL.value].Volume + self.movement_Array[
             EMovement_Index.EBT.value].Volume + self.movement_Array[EMovement_Index.EBR.value].Volume + \
@@ -379,63 +394,10 @@ class CSignalNode:
                                  self.movement_Array[EMovement_Index.SBL.value].Volume + self.movement_Array[
                                      EMovement_Index.SBT.value].Volume + self.movement_Array[
                                      EMovement_Index.SBR.value].Volume
-        # self.stage_Actual_Size = 2
-        #
-        # east_And_West_Flag = False
-        # north_And_South_Flag = False
-        # if self.movement_Array[EMovement_Index.EBL.value].Left_Turn_Treatment!= None and self.movement_Array[EMovement_Index.EBL.value].Left_Turn_Treatment.name == "prot":
-        #     self.stage_Actual_Size += 1
-        #     east_And_West_Flag = True
-        # elif self.movement_Array[EMovement_Index.WBL.value].Left_Turn_Treatment!=None and self.movement_Array[EMovement_Index.WBL.value].Left_Turn_Treatment.name == "prot":
-        #     self.stage_Actual_Size += 1
-        #     east_And_West_Flag = True
-        # if self.movement_Array[EMovement_Index.NBL.value].Left_Turn_Treatment!=None and self.movement_Array[EMovement_Index.NBL.value].Left_Turn_Treatment.name == "prot":
-        #     self.stage_Actual_Size += 1
-        #     north_And_South_Flag = True
-        # elif self.movement_Array[EMovement_Index.SBL.value].Left_Turn_Treatment!=None and self.movement_Array[EMovement_Index.SBL.value].Left_Turn_Treatment.name == "prot":
-        #     self.stage_Actual_Size += 1
-        #     north_And_South_Flag = True
-
-        # firstL = EMovement_Index
-        # firstT = EMovement_Index
-        # firstR = EMovement_Index
-        # secondL = EMovement_Index
-        # secondT = EMovement_Index
-        # secondR = EMovement_Index
-        # thridL = EMovement_Index
-        # thridT = EMovement_Index
-        # thridR = EMovement_Index
-        # fouthL = EMovement_Index
-        # fouthT = EMovement_Index
-        # fouthR = EMovement_Index
 
         if (east_And_West_Volume >= north_And_South_Volume):
-            # firstL = EMovement_Index.EBL.value
-            # firstT = EMovement_Index.EBT.value
-            # firstR = EMovement_Index.EBR.value
-            # secondL = EMovement_Index.WBL.value
-            # secondT = EMovement_Index.WBT.value
-            # secondR = EMovement_Index.WBR.value
-            # thridL = EMovement_Index.NBL.value
-            # thridT = EMovement_Index.NBT.value
-            # thridR = EMovement_Index.NBR.value
-            # fouthL = EMovement_Index.SBL.value
-            # fouthT = EMovement_Index.SBT.value
-            # fouthR = EMovement_Index.SBR.value
             self.EW_OR_NS = True
         else:
-            # firstL = EMovement_Index.NBL.value
-            # firstT = EMovement_Index.NBT.value
-            # firstR = EMovement_Index.NBR.value
-            # secondL = EMovement_Index.SBL.value
-            # secondT = EMovement_Index.SBT.value
-            # secondR = EMovement_Index.SBR.value
-            # thridL = EMovement_Index.EBL.value
-            # thridT = EMovement_Index.EBT.value
-            # thridR = EMovement_Index.EBR.value
-            # fouthL = EMovement_Index.WBL.value
-            # fouthT = EMovement_Index.WBT.value
-            # fouthR = EMovement_Index.WBR.value
             self.EW_OR_NS = False
         self.major_Approach = E_Major_Approach.EW if self.EW_OR_NS else E_Major_Approach.NS
 
@@ -486,120 +448,32 @@ class CSignalNode:
         self.stage_Actual_Size=4-len(remove_Stage_List)
         return
 
-
-
-
-
-        for movement,stage in movement_Stage_Dict.items():
-            self.movement_Array[movement.value].StageNo_in_Order.append(stage)
-        # if east_And_West_Flag:
-        #     self.movement_Array[firstL].StageNo_in_Order.append(EStage.stage1)
-        #     self.movement_Array[firstT].StageNo_in_Order.append(EStage.stage2)
-        #     self.movement_Array[firstR].StageNo_in_Order.append(EStage.stage2)
-        #     self.movement_Array[secondL].StageNo_in_Order.append(EStage.stage1)
-        #     self.movement_Array[secondT].StageNo_in_Order.append(EStage.stage2)
-        #     self.movement_Array[secondR].StageNo_in_Order.append(EStage.stage2)
-        #     if (north_And_South_Flag):
-        #         self.movement_Array[thridL].StageNo_in_Order.append(EStage.stage3)
-        #         self.movement_Array[thridT].StageNo_in_Order.append(EStage.stage4)
-        #         self.movement_Array[thridR].StageNo_in_Order.append(EStage.stage4)
-        #         self.movement_Array[fouthL].StageNo_in_Order.append(EStage.stage3)
-        #         self.movement_Array[fouthT].StageNo_in_Order.append(EStage.stage4)
-        #         self.movement_Array[fouthR].StageNo_in_Order.append(EStage.stage4)
-        #     else:
-        #         self.movement_Array[thridL].StageNo_in_Order.append(EStage.stage3)
-        #         self.movement_Array[thridT].StageNo_in_Order.append(EStage.stage3)
-        #         self.movement_Array[thridR].StageNo_in_Order.append(EStage.stage3)
-        #         self.movement_Array[fouthL].StageNo_in_Order.append(EStage.stage3)
-        #         self.movement_Array[fouthT].StageNo_in_Order.append(EStage.stage3)
-        #         self.movement_Array[fouthR].StageNo_in_Order.append(EStage.stage3)
-        # else:
-        #     self.movement_Array[firstL].StageNo_in_Order.append(EStage.stage1)
-        #     self.movement_Array[firstT].StageNo_in_Order.append(EStage.stage1)
-        #     self.movement_Array[firstR].StageNo_in_Order.append(EStage.stage1)
-        #     self.movement_Array[secondL].StageNo_in_Order.append(EStage.stage1)
-        #     self.movement_Array[secondT].StageNo_in_Order.append(EStage.stage1)
-        #     self.movement_Array[secondR].StageNo_in_Order.append(EStage.stage1)
-        #     if north_And_South_Flag:
-        #         self.movement_Array[thridL].StageNo_in_Order.append(EStage.stage2)
-        #         self.movement_Array[thridT].StageNo_in_Order.append(EStage.stage3)
-        #         self.movement_Array[thridR].StageNo_in_Order.append(EStage.stage3)
-        #         self.movement_Array[fouthL].StageNo_in_Order.append(EStage.stage2)
-        #         self.movement_Array[fouthT].StageNo_in_Order.append(EStage.stage3)
-        #         self.movement_Array[fouthR].StageNo_in_Order.append(EStage.stage3)
-        #     else:
-        #         self.movement_Array[thridL].StageNo_in_Order.append(EStage.stage2)
-        #         self.movement_Array[thridT].StageNo_in_Order.append(EStage.stage2)
-        #         self.movement_Array[thridR].StageNo_in_Order.append(EStage.stage2)
-        #         self.movement_Array[fouthL].StageNo_in_Order.append(EStage.stage2)
-        #         self.movement_Array[fouthT].StageNo_in_Order.append(EStage.stage2)
-        #         self.movement_Array[fouthR].StageNo_in_Order.append(EStage.stage2)
-
-        initialStage_Range = self.stage_Actual_Size
-        criticalStage = -1
-
-        for s in range(initialStage_Range, 1, -1):
-            checkNumber = 0
-            if criticalStage != -1:
-                for m in range(1, movementSize + 1):
-                    for so in range(len(self.movement_Array[m].StageNo_in_Order)):
-                        if self.movement_Array[m].StageNo_in_Order[so].value > criticalStage and self.movement_Array[m].Enable == True:
-                            self.movement_Array[m].StageNo_in_Order[so] = EStage(self.movement_Array[m].StageNo_in_Order[so].value - 1)
-            for m in range(1, movementSize + 1):
-                for so in range(len(self.movement_Array[m].StageNo_in_Order)):
-                    if self.movement_Array[m].Enable == True and self.movement_Array[m].StageNo_in_Order[so].value == s:
-                        checkNumber += 1
-            if checkNumber == 0:
-                self.stage_Actual_Size -= 1
-                criticalStage = s
-            else:
-                criticalStage = -1
-        for m in range(1, movementSize + 1):
-            if (self.movement_Array[m].Enable == False):
-                self.movement_Array[m].StageNo_in_Order[0] = EStage(-1)
-        for m in range(1, movementSize + 1):
-            if self.movement_Array[m].Enable == True and self.movement_Array[m].GroupNo.value == 1:
-                if self.movement_Array[self.left_Movement_Counterpart_Right_Trun_Index_Map[EMovement_Index(m)].value].Enable == True:
-                    self.movement_Array[self.left_Movement_Counterpart_Right_Trun_Index_Map[
-                        EMovement_Index(m)].value].StageNo_in_Order.insert(self.movement_Array[self.left_Movement_Counterpart_Right_Trun_Index_Map[EMovement_Index(m)].value].StageNo_in_Order[0].value - 1,
-                                                                          self.movement_Array[m].StageNo_in_Order[0])
-
-        self.Info=f"Number of Stages: {self.stage_Actual_Size}"
-        loggings.info(self.Info, 4)
-        for m in range(1, movementSize + 1):
-            for stage_o in self.movement_Array[m].StageNo_in_Order:
-                self.Info=f"StageNo of Movement({EMovement_Index(m).name}):{stage_o.name}"
-                loggings.info(self.Info,4)
-
     def Set_Saturation_Flow_Rate_Matrix(self):
-        loggings.info("Set Saturation Flow Rate Matrix", 3)
         for m in range(1, movementSize + 1):
             if self.movement_Array[m].Enable == False:
                 continue
             for so in range(len(self.movement_Array[m].StageNo_in_Order)):
                 if self.movement_Array[m].Left_Turn_Treatment.name == "prot":
-                    self.saturation_Flow_Rate_Matrix[self.movement_Array[m].StageNo_in_Order[so].value][m] = 1530 * self.movement_Array[m].Lanes * self.PHF
+                    self.saturation_Flow_Rate_Matrix[self.movement_Array[m].StageNo_in_Order[so].value][m] = saturated_flow_rate * self.movement_Array[m].Lanes * self.PHF
                 elif self.movement_Array[m].Left_Turn_Treatment.name == "perm":
                     op_Movement_Index = self.left_Movement_Opposing_Index_Map[EMovement_Index(m)]
                     op_volume = self.movement_Array[op_Movement_Index.value].Volume
                     if self.movement_Array[op_Movement_Index.value].Enable==False:
                         self.saturation_Flow_Rate_Matrix[self.movement_Array[m].StageNo_in_Order[so].value][m] =160
                     else:
-                        temp=self.f_lu * self.f_hv * op_volume * math.exp((-op_volume * 4.5 / 3600)) / (1 - math.exp(op_volume * 2.5 / 3600))
+                        temp=self.f_lu * self.f_hv * op_volume * math.exp((-op_volume * 4.5 / 3600)) / (1 - math.exp(-op_volume * 2.5 / 3600))
                         if temp>0:
                             self.saturation_Flow_Rate_Matrix[self.movement_Array[m].StageNo_in_Order[so].value][m] = temp
                         else:
-                            self.saturation_Flow_Rate_Matrix[self.movement_Array[m].StageNo_in_Order[so].value][m]=1530 *self.movement_Array[m].Lanes
+                            self.saturation_Flow_Rate_Matrix[self.movement_Array[m].StageNo_in_Order[so].value][m]=saturated_flow_rate *self.movement_Array[m].Lanes
                 else:
-                    self.saturation_Flow_Rate_Matrix[self.movement_Array[m].StageNo_in_Order[so].value][m] = 1530 *self.movement_Array[m].Lanes
-                self.Info=f"Saturation Flow Rate of Movement({EMovement_Index(m).name}) and Stage({self.movement_Array[m].StageNo_in_Order[so].name}): {self.saturation_Flow_Rate_Matrix[self.movement_Array[m].StageNo_in_Order[so].value][m]}"
-                loggings.info(self.Info,4)
+                    self.saturation_Flow_Rate_Matrix[self.movement_Array[m].StageNo_in_Order[so].value][m] = saturated_flow_rate *self.movement_Array[m].Lanes
+
 
     #The sum of the flow ratios for the critical lane groups for this phasing plan will be needed for the next section.
     # Since this phasing plan does not include any overlapping phases,
     # this value is simply the sum of the highest lane group v/s ratios for the three stages, as follows:
     def Calculate_Flow_of_Ratio_Max(self):
-        loggings.info("Calculate Max Flow Ratio ", 3)
         for s in range(1, self.stage_Actual_Size + 1):
             self.y_Max_Stage_Array[s] = 0
             for m in range(1, movementSize + 1):
@@ -617,12 +491,9 @@ class CSignalNode:
             self.y_StageMax += self.y_Max_Stage_Array[i]
 
     def Calculate_Total_Cycle_Lost_Time(self):
-        loggings.info("Calculate Total Cycle Lost Time ", 3)
         self.l_value = self.t_L * self.stage_Actual_Size
-        loggings.info(f"Total Cycle Lost Time: {self.l_value}s",4)
 
     def Calculate_the_Minimum_And_Optimal_Cycle_Length(self):
-        loggings.info("Calculate the Minimum and Optimal Cycle Length", 3)
         calculated_cycle_length=(self.l_value * self.x_c_Input) / (self.x_c_Input - self.y_StageMax)
         if self.use_reference_cycle_length:
             self.cycle_length=self.reference_cycle_length
@@ -632,75 +503,54 @@ class CSignalNode:
             else:
                 self.cycle_length=self.cycle_length_min
 
-
-        loggings.info(f"Minimum Cycle Length: {self.cycle_length}s", 4)
         self.c_Optimal = max(60, (1.5 * self.l_value + 5) / (1 - self.y_StageMax) if self.y_StageMax != 1 else (1.5 * self.l_value + 5))
-        loggings.info(f"Optimal Cycle Length: {self.c_Optimal}s", 4)
 
     def Calculate_the_x_c_Output(self):
-        loggings.info("Calculate x_c", 3)
         self.x_c_output = round((self.y_StageMax * self.cycle_length) / (self.cycle_length - self.l_value),3)
-        loggings.info(f"x_c: {self.x_c_output}", 4)
 
     def Calculate_Green_Time_for_Stages(self):
-        loggings.info("Calculate Green Time for Stages", 3)
         assigned_green_time=0
         for s in range(1, self.stage_Actual_Size + 1):
             greenTime=max(self.minGreenTime, self.y_Max_Stage_Array[s] * self.cycle_length / self.y_StageMax)
-            greenTime=min(self.reference_cycle_length-15-assigned_green_time,greenTime)
+            # greenTime=min(self.reference_cycle_length-15-assigned_green_time,greenTime)
+            greenTime = min(self.reference_cycle_length - assigned_green_time, greenTime)
             self.green_Time_Stage_Array[s] = greenTime
             assigned_green_time+=greenTime
             self.effective_Green_Time_Stage_Array[s] = self.green_Time_Stage_Array[s] - self.t_L + self.t_Yellow + self.t_AR
             self.ratio_of_Effective_Green_Time_to_Cycle_Length_Array[s] = self.effective_Green_Time_Stage_Array[s] / self.cycle_length
 
     def Printing_Green_Time_for_Stages(self):
-        loggings.info("Print Green Time for Stages", 3)
         self.cumulative_Green_Start_Time_Stage_Array[1] = 0
         self.cumulative_Green_End_Time_Stage_Array[1] = self.green_Time_Stage_Array[1]
-        loggings.info(f"Green Time of Stage 1: {round(self.green_Time_Stage_Array[1],1)}s",4)
-        loggings.info(f"Start Green Time of Stage 1: {round(self.cumulative_Green_Start_Time_Stage_Array[1],1)}s",4)
-        loggings.info(f"End Green Time of Stage 1: {round(self.cumulative_Green_End_Time_Stage_Array[1],1)}s",4)
 
         for i in range(2, self.stage_Actual_Size + 1):
             self.cumulative_Green_Start_Time_Stage_Array[i] = self.cumulative_Green_End_Time_Stage_Array[i - 1]
             self.cumulative_Green_End_Time_Stage_Array[i] = self.cumulative_Green_Start_Time_Stage_Array[i] + self.green_Time_Stage_Array[i]
-            loggings.info(f"Green Time of Stage {i}: {round(self.green_Time_Stage_Array[i],1)}s", 4)
-            loggings.info(f"Start Green Time of Stage {i}: {round(self.cumulative_Green_Start_Time_Stage_Array[i],1)}s", 4)
-            loggings.info(f"End Green Time of Stage {i}: {round(self.cumulative_Green_End_Time_Stage_Array[i],1)}s", 4)
 
         self.cumulative_Effective_Green_Start_Time_Stage_Array[1] = 0
         self.cumulative_Effective_Green_End_Time_Stage_Array[1] = self.effective_Green_Time_Stage_Array[1]
-        loggings.info(f"Effective Green Time of Stage 1: {round(self.effective_Green_Time_Stage_Array[1],1)}s",4)
-        loggings.info(f"Start Effective Green Time of Stage 1: {round(self.cumulative_Effective_Green_Start_Time_Stage_Array[1],1)}s",4)
-        loggings.info(f"End Effective Green Time of Stage 1: {round(self.cumulative_Effective_Green_End_Time_Stage_Array[1],1)}s",4)
+
         for i in range(2, self.stage_Actual_Size + 1):
             self.cumulative_Effective_Green_Start_Time_Stage_Array[i] = self.cumulative_Effective_Green_End_Time_Stage_Array[i - 1]
             self.cumulative_Effective_Green_End_Time_Stage_Array[i] = self.cumulative_Effective_Green_Start_Time_Stage_Array[i] + self.effective_Green_Time_Stage_Array[i]
-            loggings.info(f"Effective Green Time of Stage {i}: {round(self.effective_Green_Time_Stage_Array[i],1)}s", 4)
-            loggings.info(f"Start Effective Green Time of Stage {i}: {round(self.cumulative_Effective_Green_Start_Time_Stage_Array[i],1)}s", 4)
-            loggings.info(f"End Effective Green Time of Stage {i}: {round(self.cumulative_Effective_Green_End_Time_Stage_Array[i],1)}s", 4)
+
 
     def Calculate_Capacity_And_Ratio_V_over_C(self):
-        loggings.info("Calculate Capacity and V over C Ratio", 3)
         for s in range(1, self.stage_Actual_Size + 1):
             for m in range(1, movementSize + 1):
                 for so in range(len(self.movement_Array[m].StageNo_in_Order)):
                     if self.saturation_Flow_Rate_Matrix[s][m] != 0 and self.movement_Array[m].Enable and self.movement_Array[m].StageNo_in_Order[so].value == s:
                         self.capacity_by_Stage_and_Movement_Matrix[s][m] = self.saturation_Flow_Rate_Matrix[s][m] * self.ratio_of_Effective_Green_Time_to_Cycle_Length_Array[s]
-                        self.Info=f"a. Capacity of Stage({s}) and Movement({EMovement_Index(m).name}): {round(self.capacity_by_Stage_and_Movement_Matrix[s][m],1)}"
-                        loggings.info(self.Info,4)
+
                         self.v_over_C_by_Stage_and_Movement_Matrix[s][m] = self.movement_Array[m].Volume / self.capacity_by_Stage_and_Movement_Matrix[s][m]
-                        self.Info=f"b. V/C of Stage({s}) and Movement({EMovement_Index(m).name}): {round(self.v_over_C_by_Stage_and_Movement_Matrix[s][m],1)}"
-                        loggings.info(self.Info,4)
+
 
     def Calculate_Signal_Delay(self, nodeID):
-        loggings.info("Calculate Signal Delay", 3)
         for s in range(1, self.stage_Actual_Size + 1):
             for m in range(1, movementSize + 1):
                 for so in range(len(self.movement_Array[m].StageNo_in_Order)):
                     if self.saturation_Flow_Rate_Matrix[s][m] != 0 and self.movement_Array[m].Enable and self.movement_Array[m].StageNo_in_Order[so].value == s:
-                        self.average_Uniform_Delay_Matrix[s][m] = (0.5 * self.capacity_by_Stage_and_Movement_Matrix[s][
-                            m] * pow((1 - self.ratio_of_Effective_Green_Time_to_Cycle_Length_Array[s]), 2)) / (1 -self.v_over_C_by_Stage_and_Movement_Matrix[s][m] *self.ratio_of_Effective_Green_Time_to_Cycle_Length_Array[s])
+                        self.average_Uniform_Delay_Matrix[s][m] = (0.5 * self.capacity_by_Stage_and_Movement_Matrix[s][m] * pow((1 - self.ratio_of_Effective_Green_Time_to_Cycle_Length_Array[s]), 2)) / (1 -self.v_over_C_by_Stage_and_Movement_Matrix[s][m] *self.ratio_of_Effective_Green_Time_to_Cycle_Length_Array[s])
                         self.approach_Total_Delay_Array[self.movement_Array[m].DirectionNo.value] += self.movement_Array[m].Volume * self.average_Uniform_Delay_Matrix[s][m]
                         self.approach_Total_Volume_Array[self.movement_Array[m].DirectionNo.value] += self.movement_Array[m].Volume
         for d in range(1, directionSize + 1):
@@ -710,11 +560,9 @@ class CSignalNode:
             self.intersection_Total_Delay += self.approach_Average_Delay_Array[d] * self.approach_Total_Volume_Array[d]
             self.intersection_Total_Volume += self.approach_Total_Volume_Array[d]
         self.intersection_Average_Delay = self.intersection_Total_Delay / self.intersection_Total_Volume
-        loggings.info(f"Total Delay of Signal Node({nodeID}): {round(self.intersection_Total_Delay,1)}",4)
-        loggings.info(f"Average Delay of Signal Node({nodeID}): {round(self.intersection_Average_Delay,1)}",4)
+
 
     def Determine_Signal_Intersection_LOS(self, nodeID):
-        loggings.info("Determine Signal Intersection LOS", 3)
         if self.intersection_Total_Delay <= 10:
             self.LOS = "A"
         elif self.intersection_Total_Delay <= 20:
@@ -727,7 +575,6 @@ class CSignalNode:
             self.LOS = "E"
         else:
             self.LOS = "F"
-        loggings.info(f"LOS of Signal Node({nodeID}): {self.LOS}",4)
 
 
 class CNode:
@@ -761,15 +608,13 @@ class CLink:
         self.length = 0.001
 
 def set_working_directory(data_Set_Path):
-    loggings.info("Set Working Directory")
     os.getcwd()
     if not data_Set_Path=="":
         os.chdir(data_Set_Path)
         path=data_Set_Path
     else:
         path="root"
-    loggings.info("Dataset_Path:" + path)
-    loggings.info("Start Iterating", 0)
+
 # Instance
 mainModual = CMainModual()
 data_Set_Path = r''
@@ -778,33 +623,30 @@ set_working_directory(data_Set_Path)
 def Configuration():
     # curPath = os.path.dirname(os.path.realpath(__file__))
     working_directory = os.getcwd()
-    loggings.info(f"Working Directory:{working_directory}")
-    templateFullFileName = os.path.join(".signal4gmns", "config.yaml")
-    template_yaml = YamlHandler(templateFullFileName)
+    #templateFullFileName = os.path.join(".signal4gmns", "config.yaml")
+    #template_yaml = YamlHandler(templateFullFileName)
+    template_yaml = YamlHandler("")
     yamlPath = os.path.join(working_directory, "config.yaml")
     bool_yaml_existence_flag = os.path.exists(yamlPath)
 
     if not bool_yaml_existence_flag:
-        template_yaml_data = template_yaml.get_ymal_data()
+        template_yaml_data_dic = template_yaml.get_default_config_dic()
         config_yaml = YamlHandler(yamlPath)
-        config_yaml.write_yaml(template_yaml_data)
+        config_yaml.write_yaml(template_yaml_data_dic)
 
     config_yaml = YamlHandler(yamlPath)
     _config_Parameters = config_yaml.get_ymal_data()
     return  _config_Parameters
 
 def load_movement_data_and_volume():
-    loggings.info("Load Data")
     mainModual.g_number_of_nodes = 0
     mainModual.g_number_of_links = 0
     mainModual.g_number_of_movements=0
     internal_node_seq_no = 0
-    loggings.info("Configuration", 2)
     _config_Parameters=Configuration()
 
-    loggings.info("Reading file movement.csv...", 2)
-    parser_movement = pd.read_csv("movement.csv")
-    parser = pd.read_csv("node.csv",converters = {"osm_node_id":str})
+    parser_movement = pd.read_csv(movementFileName)#"movement.csv"
+    parser = pd.read_csv(macroNodeFileName,converters = {"osm_node_id":str})#"node.csv"
 
     pass_node_list=[]
 
@@ -845,7 +687,6 @@ def load_movement_data_and_volume():
                     if ctrl_type!="signal" and ctrl_type!="1":
                         pass_node_list.append(main_node_id)
                         continue
-                    loggings.info("Reading file node.csv...", 2)
                     xcoord=row["x_coord"].values[0]
                     ycoord = row["y_coord"].values[0]
                     rcl=str(row["reference_cycle_length"].values[0])
@@ -874,18 +715,15 @@ def load_movement_data_and_volume():
                 mainModual.g_number_of_movements +=1
     g_info_String = "Number of Nodes = "
     g_info_String += str(mainModual.g_number_of_nodes)
-    loggings.info(g_info_String, 3)
 
     g_info_String = "Number of Movement = "
     g_info_String += str(mainModual.g_number_of_movements)
-    loggings.info(g_info_String, 3)
 
 
 
 
 
 def Get_Signal_Timing_Movement_Info():
-    loggings.info("Generating info for signal_timing_phase.csv and signal_phase_mvmt.csv", 2)
     signal_timing_phase_list = []
     signal_phase_mvmt_list = []
     for node_id, sn in g_node_map.items():
@@ -920,9 +758,11 @@ def Get_Signal_Timing_Movement_Info():
                     # capacity = int(sn.green_Time_Stage_Array[StageNo.value] * 1800.0 / 3600.0)
                     capacity = abs(sn.capacity_by_Stage_and_Movement_Matrix[StageNo.value][m])
                     v_over_c = abs(sn.v_over_C_by_Stage_and_Movement_Matrix[StageNo.value][m])
-                    greenTime = int(sn.green_Time_Stage_Array[StageNo.value])
+                    # greenTime = int(sn.green_Time_Stage_Array[StageNo.value])
+                    greenTime=end_green_time_in_sec- start_green_time_in_sec
                     osm_web_address=f"http://openstreetmap.org/node/{node_id}"
-                    redTime = int(cycle_length_in_sec - greenTime)
+                    #redTime = int(cycle_length_in_sec - greenTime)
+                    redTime = int(cycle_length_in_sec - end_green_time_in_sec)
                     NEMA_Phase=sn.movement_Array[m].NEMA_Phase
                     ring=sn.movement_Array[m].Ring
 
@@ -979,7 +819,6 @@ def Get_Signal_Timing_Movement_Info():
     return signal_timing_phase_list, signal_phase_mvmt_list
 
 def output_signal_phasing_files():
-    loggings.info("Output signal_phase files")
     signal_timing_phase_list, signal_phase_mvmt_list=Get_Signal_Timing_Movement_Info()
 
     signal_timing_phase_list_df=pd.DataFrame(signal_timing_phase_list,
@@ -990,8 +829,8 @@ def output_signal_phasing_files():
                                                       "signal_phase_num",
                                                       "min_green",
                                                       "max_green",
-                                                      "green_start_time",
-                                                      "green_end_time",
+                                                      "start_green_time",#"green_start_time"
+                                                      "end_green_time", #"green_end_time"
                                                       "extension",
                                                       "clearance",
                                                       "walk_time",
@@ -1052,8 +891,8 @@ def Get_Timing_Info(merge_table):
         green_time_window=str(merge_table["green_time_window"][i])
         green_time=str(merge_table["min_green"][i])
         cycle_length_in_sec=str(merge_table["cycle_length"][i])
-        start_green_time=str(merge_table["green_start_time"][i])
-        end_green_time=str(merge_table["green_end_time"][i])
+        start_green_time=str(merge_table["start_green_time"][i])#"green_start_time"
+        end_green_time=str(merge_table["end_green_time"][i])#"green_end_time"
         walk_time=str(merge_table["walk_time"][i])
         osm_web_address=str(merge_table["osm_web_address_x"][i])
         capacity=str(merge_table["capacity"][i])
@@ -1091,7 +930,7 @@ def Get_Timing_Info(merge_table):
             geometry
         ])
     return timing_file_list
-def output_signal_timing_file():
+def output_signal_timing_file_and_modify_microLink_file():
     merge_table=Read_Signal_Timing_Phase_Movement_Files_Data()
     timing_file_list=Get_Timing_Info(merge_table)
     timing_file_list_df = pd.DataFrame(timing_file_list,
@@ -1110,8 +949,8 @@ def output_signal_timing_file():
                                                         "green_time_window",
                                                         "green_time",
                                                         "cycle_length",
-                                                        "green_start_time",
-                                                        "green_end_time",
+                                                        "start_green_time", #"green_start_time"
+                                                        "end_green_time",#"green_end_time"
                                                         "red_time",
                                                         "capacity",
                                                         "v_over_c",
@@ -1121,7 +960,27 @@ def output_signal_timing_file():
                                                         "geometry"
                                                       ])
     timing_file_list_df.to_csv("timing.csv", index=False)
-    loggings.info(" Thanks for using signal4gmns."
+
+    if not os.path.exists(microLinkBackupFileName):
+        microLinkDataFrame = pd.read_csv(microLinkFileName, dtype="str")
+        microLinkDataFrame.to_csv(microLinkBackupFileName, index=False)
+
+
+    microLinkDataFrame = pd.read_csv(microLinkBackupFileName, dtype="str")
+
+
+    part_timing_file_list_df=pd.DataFrame(timing_file_list_df,columns=["node_id","mvmt_txt_id","cycle_length","stage_no","green_time","red_time","start_green_time","end_green_time","capacity"])
+    part_timing_file_list_df.rename(columns={"node_id":"macro_node_id"},inplace=True)
+    part_timing_file_list_df.rename(columns={"capacity": "updated_capacity"}, inplace=True)
+    updatedMicroLink=pd.merge(microLinkDataFrame,part_timing_file_list_df,on=["macro_node_id","mvmt_txt_id"],how="left")
+
+
+    updatedMicroLink.to_csv(microLinkFileName, index=False)
+
+
+
+    print("Finished."
+          " Thanks for using signal4gmns."
                   " Please use your favorite social media platform to spread the word: https://pypi.org/project/signal4gmns/."
                   " Please help triage bugs.", 0)
 
@@ -1236,7 +1095,6 @@ def Input_Intermediate_Files(label_name="setting"):
         if len(movementStr) > 0 and movementStr != "nan":
             if osm_node_id !="":
                 if osm_node_id not in g_node_map.keys():
-                    loggings.info("Reading intermediate file node.csv...", 2)
                     row=parser[parser["osm_node_id"]==osm_node_id]
                     xcoord = row["x_coord"].values[0]
                     ycoord = row["y_coord"].values[0]
@@ -1274,26 +1132,15 @@ def Input_Intermediate_Files(label_name="setting"):
                     g_node_map[osm_node_id].AddMovementVolume_Additional(movementStr,stageNo_in_Order,groupNo,directionNo,movementCode)
                 g_node_map[osm_node_id].OSM_Node_ID=osm_node_id
                 mainModual.g_number_of_movements +=1
-    g_info_String = "Number of Nodes = "
-    g_info_String += str(mainModual.g_number_of_nodes)
-    loggings.info(g_info_String, 3)
-
-    g_info_String = "Number of Movement = "
-    g_info_String += str(mainModual.g_number_of_movements)
-    loggings.info(g_info_String, 3)
-
     return
 
-def display_signalNode_info():
-    loggings.info("Check the List of Signal Nodes")
-    for key in g_node_map.keys():
-        loggings.info("OSM_NODE_ID:{key}".format(key=key))
+
 #"road_link_id", "from_node_id", "to_node_id",
 #"main_node_id",
 
 
-def set_default_volumes():
-    node_data=pd.read_csv("node.csv")
+def set_reference_cycle_length():
+    node_data=pd.read_csv(macroNodeFileName)
     if "notes" not in node_data.columns:
         node_data["notes"]=""
     if "reference_cycle_length" not in node_data.columns:
@@ -1303,11 +1150,10 @@ def set_default_volumes():
         if ctrl_type=="signal" or ctrl_type=="1":
             node_data.loc[i,"notes"]="reference_cycle_length set by default"
             node_data.loc[i,"reference_cycle_length"]=90
-    node_data.to_csv("node.csv")
-    logging.info("reference_cycle_length set by default in node.csv ")
+    node_data.to_csv(macroNodeFileName,index=False)
 
-def set_reference_cycle_length():
-    movement_data = pd.read_csv("movement.csv")
+def set_default_volume():
+    movement_data = pd.read_csv(movementFileName)
     if "notes" not in movement_data.columns:
         movement_data["notes"]=""
     for i in range(len(movement_data["mvmt_txt_id"])):
@@ -1323,19 +1169,16 @@ def set_reference_cycle_length():
         # movement_data["notes"][i] = "volume set by default"
         movement_data.loc[i, "notes"] = "volume set by default"
         movement_data.loc[i, "volume"]=init_volume
-    movement_data.to_csv("movement.csv")
-    logging.info("volume set by default in movement.csv ")
+    movement_data.to_csv(movementFileName,index=False)
 
 
 
 def determine_major_approach():
-    loggings.info("Module 1: Output Location for each Signal Node")
     for osmID, signal_Node in g_node_map.items():
         signal_Node.Set_Major_Apporach()
     Output_Intermediate_Files()
 
 def select_left_turn_treatment():
-    loggings.info("Module 2: Perform Left turn treatment for each signal node")
     Input_Intermediate_Files()
     for osmID, signal_Node in g_node_map.items():
         signal_Node.Initialization()
@@ -1372,7 +1215,6 @@ def Preset_Phase_Mvmt_File():
 
 
 def estimate_signal_timing(input_preset_phase_mvmt_file:bool=False):
-    loggings.info("Module 3: Perform timing_process for each signal node")
     if input_preset_phase_mvmt_file:
         Preset_Phase_Mvmt_File()
     Input_Intermediate_Files()
